;================================================================================
;
;	
;	This file contains code for the foreground processing part of the 26C92
;	driver.  The functions herein are:
;
;	GETCHA    Gets datum from channel A.
;	GETCHB    Gets datum from channel B.
;	PUTCHA    Puts datum to channel A.
;	PUTCHA    Puts datum to channel A.
;	CHANBCTL  Enables or disables channel B.
;
;	In the above, "datum" is synonomous with "byte."  Each function has com-
;	mentary that describes the calling procedure & returns.
;
;	These functions have been rewritten to use channel indexing instead of
;	hard-coded DUART port addresses.  The goal is to make a portable driver
;	that will (we hope!) work with any of the NXP multichannel UARTs.
;
;	The GET... functions read their respective channels' buffers, returning
;	immediately regardless of the availability of data.  Data availability
;	is indicated by the MPU's carry bit, WHICH MUST BE TESTED AFTER EACH AC-
;	CESS TO DETERMINE IF A DATUM WAS RETURNED!
;
;	The PUT... functions write their respective channels' buffers & will not
;	return until the datum has been accepted & buffered.  Blocking will oc-
;	cur if the buffer is full & the interrupt part of the driver cannot load
;	any more data into the 26C92's transmit FIFO.  The most likely reason
;	for this situation is that the remote device has stopped transmission by
;	deasserting the channel's clear-to-send (CTS) signal.  Blocking will al-
;	so occur if IRQs have been disabled, as the interrupt part of the driver
;	will no longer be executed.
;
;	The CHANBCTL ("channel B control") function may be used to enable or
;	disable channel B's receiver as required.  26C92 setup doesn't enable
;	this channel's receiver.  The transmitter is automatically disabled when
;	the buffer is empty.
;	
;
;================================================================================
;
;getcha: GET DATUM FROM TIA-232 CHANNEL A
;
;	
;	Preparatory Ops: NONE
;
;	Returned Values: .A: datum or entry value
;	                 .B: entry value
;	                 .X: entry value
;	                 .Y: entry value
;
;	MPU Flags: NVmxDIZC
;	           ||||||||
;	           |||||||+> 0: datum available
;	           |||||||      1: no datum
;	           |||++++> entry values
;	           ||+> 1
;	           ++> entry values
;
;	Example: JSR GETCHA
;	         BCS NODATUM
;	         STA DATUM
;	
;
getcha   php                   ;save register sizes
         longr                 ;16 bit registers
         phx                   ;save
         ldxw c92_chax         ;channel index
;
;================================================================================
;
;getchcom: GET DATUM FROM TIA-232 CHANNEL
;
;	
;	DO NOT directly call this function, as the stack will become unbalanced.
;	Only use the GETCHx functions, where x is the channel ID (A, B, etc.).
;	
;
getchcom phy                   ;preserve...
         pha                   ;remaining registers
         phb
         pha                   ;create workspace
;
;
.buf     =1                    ;buffer base address
.dbreg   =.buf+s_ptr           ;DB
.areg    =.dbreg+s_mpudbx      ;.C
.yreg    =.areg+s_word         ;.Y
.xreg    =.yreg+s_word         ;.X
.sreg    =.xreg+s_word         ;SR
;
;
         shortr
;
;	
;	Uncomment the following code if the data bank used during TIA-232 proc-
;	essing is different than the calling process' DB.
;	
;
;         lda #kerneldb
;         pha
;         plb                   ;set data bank
;
         lda e32getrx,x        ;RxD buffer "get" index
         cmp e32putrx,x        ;RxD buffer "put" index
         beq .0000010          ;same, buffer empty
;
         txa                   ;make a...
         asl                   ;table offset...
         tay                   ;from channel index
         longa
         lda e32rbtab,y        ;get & set...
         stas .buf             ;buffer base address
         shorta
         ldy e32getrx,x        ;RxD buffer "get" index
         ldasi .buf            ;get from buffer &...
         stas .areg            ;overwrite .C's stack copy
         tya                   ;adjust buffer...
         ina                   ;"get" index
         and #e32xmask         ;wrap to buffer boundary
         sta e32getrx,x        ;new buffer "get" index
         clc                   ;datum avaliable
;
.0000010 ldas .sreg            ;SR stack copy
         and #sr_car_i         ;clear carry
         bcc .0000020          ;datum available
;
         ora #sr_car           ;no datum available
;
.0000020 ora #sr_amw           ;force 8 bit .A &...
         stas .sreg            ;condition SR
         longr
         pla                   ;discard workspace
         plb                   ;restore...
         pla                   ;registers
         ply
         plx
         plp
         rts
;
;================================================================================
;
;getchb: GET DATUM FROM TIA-232 CHANNEL B
;
;	
;	Preparatory Ops: NONE
;
;	Returned Values: .A: datum or entry value
;	                 .B: entry value
;	                 .X: entry value
;	                 .Y: entry value
;
;	MPU Flags: NVmxDIZC
;	           ||||||||
;	           |||||||+> 0: datum available
;	           |||||||      1: no datum
;	           |||++++> entry values
;	           ||+> 1
;	           ++> entry values
;
;	Notes: 1) See code comments in GETCHA for
;	          execution details.
;
;	Example: JSR GETCHB
;	         BCS NODATUM
;	         STA DATUM
;	
;
getchb   php                   ;save register sizes
         longr
         phx                   ;save
         ldxw c92_chbx         ;channel B index
         bra getchcom
;
;================================================================================
;
;putcha: WRITE DATUM TO TIA-232 CHANNEL A
;
;	
;	Preparatory Ops: .A: 8 bit datum
;
;	Returned Values: .A: entry value
;	                 .B: entry value
;	                 .X: entry value
;	                 .Y: entry value
;
;	MPU Flags: NVmxDIZC
;	           ||||||||
;	           |||||||+> 0
;	           ||||||+> entry value
;	           |||||+> 0
;	           +++++> entry values
;
;	Example: LDA #$41
;	         JSR PUTCHA
;
;	Notes: 1) This function will block if the buffer is full.
;	
;
putcha   php                   ;save MPU state
         longr                 ;16 bit registers
         phx                   ;save
         ldxw c92_chax         ;channel A index
;
;================================================================================
;
;putchcom: WRITE DATUM TO TIA-232 CHANNEL
;
;	
;	DO NOT directly call this function, as the stack will become unbalanced.
;	Only use the PUTCHx functions, where x is the channel ID (A, B, etc.).
;	
;
putchcom phy                   ;preserve...
         pha                   ;remaining registers
         phb
         .rept s_e32iws        ;create workspace
           pha
         .endr
;
;
.buf     =1                    ;buffer base address
.chan    =.buf+s_ptr           ;channel base address
.dbreg   =.chan+s_ptr          ;DB
.areg    =.dbreg+s_mpudbx      ;.C
.yreg    =.areg+s_word         ;.Y
.xreg    =.yreg+s_word         ;.X
.sreg    =.xreg+s_word         ;SR
;
;
         shortr
;
;	
;	Uncomment the following code if the data bank used during TIA-232 proc-
;	essing is different than the calling process' DB.
;	
;
;         lda #kerneldb
;         pha
;         plb                   ;set data bank
;
         txa                   ;make a...
         asl                   ;table offset...
         tay                   ;from channel index
         longa
         lda d92chtab,y        ;get & set...
         stas .chan            ;channel base address
         lda e32tbtab,y        ;get & set...
         stas .buf             ;buffer base address
         shorta
         lda e32puttx,x        ;TxD buffer "put" index
         tay                   ;make a copy
         ina                   ;increment for space test
         and #e32xmask         ;wrap to buffer boundary
;
.0000010 cmp e32gettx,x        ;TxD buffer "get" index
         beq .0000030          ;buffer full
;
         xba                   ;protect new index
         ldas .areg            ;get datum &...
         stasi .buf            ;buffer it
         xba                   ;recover new index &...
         sta e32puttx,x        ;set it
         lda d92tstab,x        ;TxD status flag bit
         trb e32txst           ;TxD enabled?
         beq .0000020          ;yes
;
         lda #u92crtxe         ;no
         ldy #c92_cr
         stasi .chan           ;enable transmitter
;
.0000020 longr
         .rept s_e32iws        ;discard workspace
           pla
         .endr
         plb                   ;restore...
         pla                   ;registers
         ply
         plx
         plp
         clc                   ;never an error
         rts
;
;
;	block for buffer space...
;
.0000030 wai
         bra .0000010
;
;================================================================================
;
;putchb: WRITE DATUM TO TIA-232 CHANNEL B
;
;	
;	Preparatory Ops: .A: 8 bit datum
;
;	Returned Values: .A: entry value
;	                 .B: entry value
;	                 .X: entry value
;	                 .Y: entry value
;
;	MPU Flags: NVmxDIZC
;	           ||||||||
;	           |||||||+> 0
;	           ||||||+> entry value
;	           |||||+> 0
;	           +++++> entry values
;
;	Example: LDA #$41
;	         JSR PUTCHB
;
;	Notes: 1) This function will block if the buffer is full.
;	
;
putchb   php                   ;save MPU state
         longr                 ;16 bit registers
         phx                   ;save
         ldxw c92_chbx         ;channel B index
         bra putchcom
;
;================================================================================
;
;chanbctl: TIA-232 CHANNEL B RECEIVER CONTROL
;
;	
;	Preparatory Ops : see programming example
;
;	Register Returns: .A: used
;	                  .B: entry value
;	                  .X: entry value
;	                  .Y: entry value
;	                  DB: entry value
;	                  DP: entry value
;	                  PB: entry value
;
;	MPU Flags: NVmxDIZC
;	           ||||||||
;	           |||||||+> entry value
;	           ||||||+> 0
;	           |||+++> entry values
;	           ||+> 1
;	           |+> entry value
;	           +> undefined
;
;	Examples: clc              ;disable
;	          jsr chanbctl
;
;	          sec              ;enable
;	          jsr chanbctl
;	
;
chanbctl shorta
         bcc disabchb          ;disable
;
;================================================================================
;
;enabchb: ENABLE DUART CHANNEL B RECEIVER
;
enabchb  lda #u92crrsa|u92crrxe ;enable RxD & assert RTS
         .byte $2c              ;BIT ABS opcode
;
;================================================================================
;
;disabchb: DISABLE DUART CHANNEL B RECEIVER
;
disabchb lda #u92crrsd|u92crrxd ;disable RxD & deassert RTS
;
;================================================================================
;
;chbctl: DUART CHANNEL B CONTROL
;
chbctl   sta io_acia+dr_crb    ;write to command register
         .rept 2               ;DUART reaction delay
           wai
         .endr
         rts
;
	.end
